/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <Core/Window/Window.h>

#include <QAction>
#include <QList>

#include "XmlObject.h"

class Module;

class SkywardHubCore : public QObject
{
    Q_OBJECT

public:
    static SkywardHubCore &getInstance();

    void init();

    /**
     * This method setups the core menu to the application
     * It adds themes selection function to the menu with other
     * core functionalities
     *
     * @param menu is the menu to which the actions/submenus will
     *             be added to
     */
    void setupMenu(QMenu &menu);

public slots:
    void addNewWindow();
    void addWindowFromXml(XmlObject settings);

    /**
     * This method is called when a theme from the Themes menu is selected
     *
     * @param action The action performed by the user on the QMenu. In this
     * scenario the action corresponds to the selected theme
     */
    void onThemeSelected(QAction *action);

private:
    SkywardHubCore();

    void checkDefaultFolders();
    void buildHubMenuActions();
    void loadConfigurationFromXmlObject(XmlObject settings);

    void saveConfigurationPrompt();
    QString saveConfiguration(QString configName, QString configIconPath,
                              QString description);

    void buildThemesMenuSelector();

    /**
     * This method fetches all available themes in the default themes folder
     */
    QList<QAction *> fetchAvailableThemes();

    /**
     * This method loads the stylesheet located at the given path
     * @param path The path where the stylesheet is located
     */
    void loadStyleSheet(QString path);

    /**
     * This method unsets the previous active theme
     */
    void unsetOldActiveTheme();

    void initializeActiveTheme();

    QAction *getActiveThemeAction();

    void setActiveTheme(QAction *action);

    QList<Window *> windowsList;
    QList<QAction *> hubMenuActionsList;
    QMenu *themeSelectionMenu;
    QString activeTheme;
    QString activeThemeStyleSheet;

public:
    SkywardHubCore(const SkywardHubCore &)            = delete;
    SkywardHubCore &operator=(const SkywardHubCore &) = delete;
};
