/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "Window.h"

#include <Modules/Empty/EmptyModule.h>
#include <Modules/ModulesList.h>

Window::Window(XmlObject configuration)
{
    errorDisplayer = new ErrorDisplayer(this);
    Module *module = ModulesList::getInstance().instantiateModule(
        configuration.getObjectName());

    if (module)
    {
        module->fromXmlObject(configuration);
        setChild(module);
    }
}

void Window::setChild(Module *module)
{
    setCentralWidget(module);
    module->lower();

    connect(module, &Module::replaceMe, this, &Window::replaceChild);
    connect(module, &Module::closeMe, this, &Window::closeChild);
}

void Window::replaceChild(Module *oldModule, Module *newModule)
{
    // Set the new module
    setChild(newModule);

    // Delete the old child
    delete oldModule;
}

void Window::closeChild(Module *module) { close(); }

XmlObject Window::toXmlObject()
{
    return ((Module *)centralWidget())->toXmlObject();
}

void Window::closeEvent(QCloseEvent *event)
{
    emit onWindowClosed();
    event->accept();
}